package bullybus;
import java.awt.*;
import java.applet.*;
import java.awt.event.*;
import java.io.*;
import java.util.Timer;
import java.util.TimerTask;
import java.net.*;
import java.lang.String;
import java.lang.Integer;

/**
 * "CbnGauges" is a simple applet, used to display a range of Gauges
 * using a "meter" format.
 * <P>
 * Accessible Attributes:
 * <PRE>
 * BusID:  Bus identification <color><number>
 * </PRE>
 * <P>
 * Attributes not publically accessible:
 * <PRE>
 * AirFlow:             Gauge displaying air flow in gm/s.
 * CoolantTemperature:  Gauge displaying coolant temperature in F.
 * Speed:               Gauge displaying speed in MPH.
 * RPM:                 Gauge displaying rpm in revolutions/sec.
 * TPS:                 Gauge dislaying throttle position percentage.
 * EngineLoad:          Gauge displaying engine load percentage.
 * GPS:                 GaugeGPS displaying bus ID and GPS coordinates.
 * Timer:               Timer used to acquire new performance and GPS data.
 * URL:                 URL used to access servlet which retrieves gauge or gps data
 * </PRE>
 * <P>
 * @author Will Jenkins
 * @version 1.0 07/30/2004
 *
 */
public class CbnGauges extends Panel {
    
    private Gauge airFlow;
    private Gauge coolTemp;
    private Gauge speed;
    private Gauge rpm;
    private Gauge tps;
    private Gauge engLoad;
    private GaugeGPS gps;
    
    private Panel airFlowPanel;
    private Panel coolPanel;
    private Panel speedPanel;
    private Panel rpmPanel;
    private Panel tpsPanel;
    private Panel loadPanel;
    
    private Panel gaugesPanel;
    private Panel gpsPanel;
    
    private Panel grid1, grid2;
    
    private Timer timer;
    
    private URL url = null;
    private URLConnection urlConn;
    
    private String busID;
    private int busIdNum;
    
    private CbnGaugeDataSet dSet;
    private CbnBuses buses;
    
    /**
     * Sets the initial gauges for a particular bus ID
     * @param b     Bus ID <color><number>
     */
    public CbnGauges(String b) {
	busID = b;
	if(busID.equals("38t")) {
	    busIdNum = 0;
	}
        else if(busID.equals("1104")) {
	    busIdNum = 1;
    	}
	else if(busID.equals("1003")) {
	    busIdNum = 2;
        }
	else if(busID.equals("0903")) {
	    busIdNum = 3;
	}
	else if(busID.equals("0898")) {
	    busIdNum = 4;
	}
	setup();
    }

    /**
     * Sets the initial gauges for a particular bus ID
     */
    private void setup() {

	setLayout(new BorderLayout());
	setBackground(new Color(226,226,199));
	
	grid1 = new Panel(new GridLayout(2,0));
	grid2 = new Panel(new GridLayout(2,0));
	
	gaugesPanel = new Panel(new FlowLayout(1,0,0));
	gpsPanel = new Panel(new FlowLayout(1,0,0));
	
	airFlowPanel = new Panel(new BorderLayout());
	coolPanel = new Panel(new BorderLayout());
	speedPanel = new Panel(new BorderLayout());
	rpmPanel = new Panel(new BorderLayout());
	tpsPanel = new Panel(new BorderLayout());
	loadPanel = new Panel(new BorderLayout());
	
	airFlow = new Gauge(0, 656, (float)1.0,  (float)1.0, "Air Flow", "gm/s");
	airFlow.setSize(136,136);
	
	coolTemp = new Gauge(0, 260, (float)1.0,  (float)1.0, "Coolant Temp", "F");
	coolTemp.setSize(136,136);
	
	speed = new Gauge(0, 120, (float)1.0, (float)1.0, "Speed", "MPH");
	speed.setSize(272,272);
	
	rpm = new Gauge(0, 100, (float)1.0, (float)1.0, "RPM", "x100");
	rpm.setSize(272,272);
	
	tps = new Gauge(0, 100, (float)1.0, (float)1.0, "TPS", "%");
	tps.setSize(136,136);
	
	engLoad = new Gauge(0, 100, (float)1.0, (float)1.0, "Engine Load", "%");
	engLoad.setSize(136,136);
	
	airFlowPanel.add("Center",airFlow);
	coolPanel.add("Center",coolTemp);
	speedPanel.add("Center",speed);
	rpmPanel.add("Center",rpm);
	tpsPanel.add("Center",tps);
	loadPanel.add("Center",engLoad);
	
	grid1.add(airFlowPanel);
	grid1.add(coolPanel);
	grid2.add(tpsPanel);
	grid2.add(loadPanel);
	
	gaugesPanel.add(grid1);
	gaugesPanel.add(speedPanel);
	gaugesPanel.add(rpmPanel);
	gaugesPanel.add(grid2);
	
	gps = new GaugeGPS(busID);
	gps.setSize(816,16);
	gps.setBackground(Color.gray);
	
	gpsPanel.add(gps);
	
	add("South",gpsPanel);
	add("North",gaugesPanel);

	//Timer to get gauge and GPS data every second
	timer = new Timer();
	timer.schedule(new Redraw(), 0, (int)(1*1000));
    }

    /**
     * At update(); draw into an off-screen image and put up the whole
     * image.  This clears up any flickering.
     */ 
    public void update(Graphics g) {
        Image osImg = createImage(getWidth(), getHeight());
        osImg.getGraphics().fillRect(0,0,getWidth(), getHeight());
        paint(osImg.getGraphics());
        g.drawImage(osImg,0,0,null);
    };
    
    class Redraw extends TimerTask
    {
	/**
	 * Timed function which gets the gauge and GPS data every second
	 */
        public void run()
        {
	    getGauges();
	    speed.setCurrent(dSet.getData(busIdNum).getSpeed());
	    rpm.setCurrent(dSet.getData(busIdNum).getRpm());
	    tps.setCurrent(dSet.getData(busIdNum).getTps());
	    engLoad.setCurrent(dSet.getData(busIdNum).getEngLoad());
	    airFlow.setCurrent(dSet.getData(busIdNum).getIntakePres());
	    coolTemp.setCurrent(dSet.getData(busIdNum).getCoolantTemp());
	    getGPS();
	    gps.setCurrentGPS(buses.getBus(busIdNum).getGpsN(),buses.getBus(busIdNum).getGpsW());
	}
    }

    /**
    * Method for getting gauge data from the database
    * Uses a servlet to make the connection
    */ 
    private void getGauges()
    {
	URL myUrl;
	URLConnection conn;
	InputStream ins=null;
	ObjectInputStream objIns;
	int numOfTimes = 0;
	int i = 0;
	double n,w;
	try{
	    long time1 = System.currentTimeMillis();
	    myUrl = new URL("http://bullybus.its.msstate.edu:8080/cbn/servlet/cbnDbGetGauges");
	    conn =myUrl.openConnection();
	    conn.setDoOutput(true);
	    conn.setDoInput(true);
	    conn.setUseCaches(false);
	    conn.setDefaultUseCaches(false);
	    conn.setRequestProperty("Content-Type",
				    "application/octet-stream");
	    
	    try{   
		ins = conn.getInputStream();
		objIns = new ObjectInputStream(ins);		
		
		dSet = (CbnGaugeDataSet)objIns.readObject();
		
		long time2 = System.currentTimeMillis();
		long elapsedTime= time2 - time1;
		
	    }catch(Exception e){   
		//System.out.println(" EXCEPTION "); 
		//e.printStackTrace();          
		numOfTimes++; 
		if( numOfTimes > 3) //max attempts  
		    throw e;  
	    }
	    
	    myUrl = null;
	    conn=null;	    
	    
	}catch(Exception e){
	    //e.printStackTrace();
	}
    }

    /**
    * Method for getting bus data from the database
    * Uses a servlet to make the connection
    */ 
    private void getGPS()
    {
	URL myUrl;
	URLConnection conn;
	InputStream ins=null;
	ObjectInputStream objIns;
	int numOfTimes = 0;
	int i = 0;
	double n,w;
	try{
	    long time1 = System.currentTimeMillis();
	    myUrl = new URL("http://bullybus.its.msstate.edu:8080/cbn/servlet/cbnDbGetBuses");
	    conn = myUrl.openConnection();
	    conn.setDoOutput(true);
	    conn.setDoInput(true);
	    conn.setUseCaches(false);
	    conn.setDefaultUseCaches(false);
	    conn.setRequestProperty("Content-Type",
				    "application/octet-stream");
	    
	    try{   
		ins = conn.getInputStream();
		objIns = new ObjectInputStream(ins);		
		
		buses = (CbnBuses)objIns.readObject();
		
		long time2 = System.currentTimeMillis();
		long elapsedTime= time2 - time1;
		
	    }catch(Exception e){   
		//System.out.println(" EXCEPTION "); 
		//e.printStackTrace();          
		numOfTimes++; 
		if( numOfTimes > 3) //max attempts  
		    throw e;  
	    }
	    
	    myUrl = null;
	    conn=null;	    
	    
	}catch(Exception e){
	    //e.printStackTrace();
	}
    }   
}
