/*
 * @(#)Gauge.java        2.0 01/25/2005
 *
 * Copyright (c) 2004 CAVS, All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and
 * without fee is hereby granted.
 *
 */
package bullybus;
import java.awt.*;
import java.awt.image.*;

/**
 * "Gauge" is a simple control, used to display a range of values
 * using a "meter" format.
 * <P>
 * Accessible Attributes:
 * <PRE>
 * MinimumValue:       Low end of the scale
 * MaximumValue:       High end of the scale
 * CurrentValue:       Current setting (where the Pointer points)
 * CriticalThreshold:  Percentage of MaxValue at which we start drawing using the Critical color
 * WarningThreshold:   Percentage of MaxValue at which we start drawing using the Warning color
 * Legend:             What we're measuring (e.g., Speed)
 * Units:              What units we're using (e.g., MPH)
 * </PRE>
 * <P>
 * Attributes not yet publically accessible:
 * <PRE>
 * MinimumAngle:   Angle at which Minimum Value will be drawn
 * MaximumAngle:   Angle at which Maximum Value will be drawn
 * Margin:         Distance between the Scale and the Component's bounds() rect
 * Separation:     Distance between Gauge subobjects (e.g, between Pointer and Scale)
 * CriticalColor:  Color used to draw the Critical Section of the Scale(e.g., Color.red)
 * WarningColor:   Color used to draw the Warning Section of the Scale
 * NormalColor:    Color used to draw the Normal Section of the Scale
 * InternalColor:  Color used to draw the interior of the Scale
 * PenRadius:      Used to determine thickness of Pointer and Scale
 * Draw3D:         Should the background be drawn as a raised rectangle?
 * TickSize:       How long (in pixels) is a BigTick?
 * BigTick:        Interval starting at Minimum at which BigTicks will be drawn
 * LittleTick:     Interval starting at Minimum at which LittleTicks will be drawn
 * </PRE>
 * <P>
 * @author Will Jenkins
 * @version 2.0 01/25/2005
 *
 */

public class Gauge extends Canvas {

    /**
     * Low end of the scale
     */
    private int minVal;
    /**
     * High end of the scale
     */
    private int  maxVal;
    /**
     * Current setting (where the Pointer points)
     */
    private int currVal;
    /**
     * Percentage of MaxValue at which we start drawing using the Warning color
     */
    private double warnPercent;
    /**
     * Percentage of MaxValue at which we start drawing using the Critical color
     */
    private double critPercent;
    /**
     * What we're measuring (e.g., Speed)
     */
    private String legend;
    /**
     * What units we're using (e.g., MPH)
     */
    private String units;
    /**
     * Color used to draw the Warning Section of the Scale
     */
    private Color warnColor;
    /**
     * Color used to draw the Critical Section of the Scale
     */
    private Color critColor;
    /**
     * Color used to draw the interior of the Scale
     */
    private Color internalColor;
    /**
     * Color used to draw the Normal Section of the Scale
     */
    private Color normalColor;
    /**
     * Distance between Gauge subobjects (e.g, between Pointer and Scale)
     */
    private int	separation = 5;
    /**
     * Distance between the Scale and the Component's bounds() rect
     */
    private int margin = separation;
    /**
     * Angle at which Minimum Value will be drawn
     */
    private double minAngleRads = Math.PI;
    /**
     * Angle at which Maximum Value will be drawn
     */
    private double maxAngleRads = 0.0;
    /**
     * Used to determine thickness of Pointer and Scale
     */
    private int	penRadius = 1;
    /**
     * Should the background be drawn as a raised rectangle?
     */
    private boolean draw3D = true;
    /**
     * How long (in pixels) is a BigTick?
     */
    private int	tickSize = 8;
    /**
     * Interval starting at Minimum at which BigTicks will be drawn
     */
    private int	bigTick = 10;
    /**
     * Interval starting at Minimum at which LittleTicks will be drawn
     */
    private int	littleTick = 5;
    
    /**
     * Computed attributes (derived from Base or elsewhere)
     */
    private int	warnVal, critVal;
    private String maxStr, minStr;
    private Dimension minSize, maxSize, legendSize, unitsSize, currValSize;
    private Point pivotLoc;
    private int pointerLen;
    private int minAngleDegs, maxAngleDegs;
    private Rectangle scaleRect;
    private FontMetrics	myMetrics = null;
    private int	halfTick = (int)Math.round(tickSize/2.0);
    
    /**
     * Constructs a Gauge with the specified Min, Max, Legend, and Units values.
     * @param min		Minimum Value
     * @param max		Maximum Value
     * @param warn		Percentage of Max of start of Warning area
     * @param crit		Percentage of Max of start of Critical area
     * @param newLegend		Gauge Legend
     * @param newUnits		Gauge Units
     */
    public Gauge(int min, int max, float warn, float crit, String newLegend, String newUnits) {
	setMinimum(min);
	setMaximum(max);
	setCurrent(min);
	
	warnPercent = warn;
	critPercent = crit;
	warnVal = (int)Math.round(maxVal*warnPercent);
	critVal = (int)Math.round(maxVal*critPercent);
	
	setLegend(newLegend);
	setUnits(newUnits);
	init();
    };
    
    /**
     * Degrees-to-Radians
     */
    private double d2r (int degs) {
	return Math.round((degs/180)*Math.PI);
    };
    
    /**
     * Radians-to-Degrees
     */
    private int r2d (double rads) {
	return (int)Math.round((rads/Math.PI)*180);
    };
    
    /**
     * Based on current font, determine dimensions of all strings.
     * Reset PointerLen here.
     */
    private void setStringMetrics() {
	if (myMetrics==null) return;
	
	minSize = new Dimension(myMetrics.stringWidth(minStr),
				myMetrics.getHeight());
	maxSize = new Dimension(myMetrics.stringWidth(maxStr),
				myMetrics.getHeight());
	legendSize = new Dimension(myMetrics.stringWidth(legend),
				   myMetrics.getHeight());
	unitsSize = new Dimension(myMetrics.stringWidth(units),
				  myMetrics.getHeight());
	currValSize = new Dimension(myMetrics.stringWidth(String.valueOf(currVal)),
				    myMetrics.getHeight());
	margin += myMetrics.getHeight();
	pointerLen = (int)Math.round(getWidth()/2.0) - margin - separation;
    };
    /**
     * Return rect into which the scale-arc will be drawn
     */
    private Rectangle findScaleRect() {
	return new Rectangle(margin, pivotLoc.y - pointerLen, 2*pointerLen, pointerLen);
    };
    
    /**
    * Determine locations of Pivot and Scale, and len of Pointer
    * Called after any size-change
    */
    private void partition() {
	pivotLoc = new Point( getWidth()/2, getHeight()/2 );
	pointerLen = (int)Math.round(getWidth()/2.0) - margin - separation;
	scaleRect = findScaleRect();
    };
    
    /**
     * init() sets Guage's attributes to a "sane" set of initial values.
     * Defaults are:
     *  CriticalColor   = Color.red
     *  WarningColor    = Color.yellow
     *  InternalColor   = Color.black
     *  NormalColor     = getForeground()
     *  ForegroundColor = Color.green
     *  BackgroundColor = Color.gray
     *  Font            = "Courier", BOLD, 18
     */
    private void init() {
	minStr = String.valueOf(minVal);
	maxStr = String.valueOf(maxVal);
	
	setStringMetrics();
	
	warnColor = Color.yellow;
	critColor = Color.red;
	internalColor = new Color(255,253,238);
	
	partition();
	minAngleDegs = r2d(minAngleRads);
	maxAngleDegs = r2d(maxAngleRads);
	
	setForeground(new Color(137,32,52));
	normalColor = getForeground();
	setBackground(new Color(226,226,199));
	setFont(new Font("Courier", Font.BOLD, 16));	
    };
    
    /**
     * Change the Gauge's Legend (what this Gauge is measuring; e.g., Speed or Swap Rate)
     * @param newLegend	Legend's new value
     */
    public void setLegend(String newLegend) {
	legend = newLegend;
	if (myMetrics!=null) {
	    legendSize = new Dimension(myMetrics.stringWidth(legend),
				       myMetrics.getHeight());
	}
	repaint();
    };
    
    /**
     * Change the Gauge's Units (what Value is measured in; e.g., KPH or PPM)
     * @param newUnits	Units's new value
     */
    public void setUnits(String newUnits) {
	units = newUnits;
	if (myMetrics!=null) {
	    unitsSize = new Dimension(myMetrics.stringWidth(units),
				      myMetrics.getHeight());
	}
	repaint();
    };
    
    /**
     * Change the Gauge's Minimum Value
     * @param min New Minimum
     */
    public void setMinimum(int min) {
	minVal = min;
	minStr = String.valueOf(minVal);
	if (myMetrics!=null) {
	    minSize = new Dimension(myMetrics.stringWidth(minStr),
				    myMetrics.getHeight());
	}
	repaint();
    };
    
    public int getMinimum() {
	return minVal;
    };
    
    /**
     * Change the Gauge's Maximum Value
     * @param max New Maximum
     */
    public void setMaximum(int max) {
	maxVal = max;
	maxStr = String.valueOf(maxVal);
	if (myMetrics!=null) {
	    maxSize = new Dimension(myMetrics.stringWidth(maxStr),
				    myMetrics.getHeight());
	}
	warnVal = (int)Math.round(maxVal*warnPercent);
	critVal = (int)Math.round(maxVal*critPercent);
	repaint();
    };
    
    public int getMaximum() {
	return maxVal;
    };
    
    /**
     * Change the Gauge's Current Value (the value the pointer points at)
     * @param newCurr New Current
     */
    public void setCurrent(int newCurr) {
	currVal = newCurr;
	repaint();
    };
    /**
     * Change the Gauge's Current Value (the value the pointer points at)
     */
    public int getCurrent() {
	return currVal;
    };
    
    /**
     * Change the Gauge's Warning Threshold.  This is the percentage of the Maximum Value
     * at which the Warning Region begins.
     * @param newWarnPcnt	New Warning Threshold
     */
    public void setWarningThreshold(float newWarnPcnt) {
	warnPercent = newWarnPcnt;
	warnVal = (int)Math.round(maxVal*warnPercent);
	repaint();
    };
    /**
     * Get the Gauge's Warning Threshold.  This is the percentage of the Maximum Value
     * at which the Warning Region begins.
     * @return arnPercent	New Warning Threshold
     */
    public double getWarningThreshold() {
	return warnPercent;
    };
    
    /**
     * Change the Gauge's Critical Threshold.  This is the percentage of the Maximum Value
     * at which the Critical Region begins.
     * @param newCritPcnt	New Critical Threshold
     */
    public void setCriticalThreshold(float newCritPcnt) {
	critPercent = newCritPcnt;
	critVal = (int)Math.round(maxVal*critPercent);
	repaint();
    };

     /**
     * Get the Gauge's Critical Threshold.  This is the percentage of the Maximum Value
     * at which the Critical Region begins.
     * @return critPercent      Critical Threshold
     */
    public double getCriticalThreshold() {
	return critPercent;
    };
    
    /**
     * At update(); draw into an off-screen image and put up the whole
     * image.  This clears up any flickering.
     */
    public void update(Graphics g) {
	Image osImg = createImage(getWidth(), getHeight());
	osImg.getGraphics().fillRect(0,0,getWidth(), getHeight());
	paint(osImg.getGraphics());
	g.drawImage(osImg,0,0,null);
    };
    
    /**
     * If Draw3d is set, draw the background as a filled 3D rect in the
     * current Background Color.  Otherwise, draw a flat rect.
     */
    private void drawBackground(Graphics g) {
	g.setColor(getBackground());
	
	if (draw3D) {
	    for (int i=0;i<=3;i++) {
		g.fill3DRect(i,i,getWidth()-(2*i), getHeight()-(2*i),true);
	    }
	} else {
	    g.fillRect(0,0, getWidth(), getHeight());
	}
    };
    
    /**
     * drawTick draws one tick-mark from center of scale-arc _INWARDS_
     * @param where		Value at which to draw the tick-mark
     * @param offset		How long to make the tick-mark
     */
    private void drawTick(Graphics g, int where, int offset) {
	int arcCenter = pointerLen+separation;
	Point startPt = mapValToPoint(where, arcCenter-offset);
	Point endPt   = mapValToPoint(where, arcCenter);
	g.setColor(getValColor(where));
	g.drawLine(startPt.x, startPt.y, endPt.x, endPt.y);
    };
    
    /**
     * drawScale draws the ScaleArc itself, and the tick-marks
     */
    private void drawScale(Graphics g) {
	
	// Draw the meter-interior.  Then draw the Arc itself.  Then, draw tickmarks.
	g.setColor(internalColor);
	g.fillArc(margin, margin,
		  getWidth() - 2*margin, getHeight() - 2*margin,
		  maxAngleDegs, (minAngleDegs-maxAngleDegs));
	
	for (int i=-penRadius;i<=penRadius;i++) {
	    // Draw the critical-color segment
	    g.setColor(critColor);
	    g.drawArc(margin+i, margin+i,
		      getWidth() - 2*margin - 2*i, getHeight() - 2*margin - 2*i,
		      maxAngleDegs,
		      (int)Math.round(minAngleDegs*(1.0-critPercent)));
	    
	    // Draw the warning-color segment
	    g.setColor(warnColor);
	    g.drawArc(margin+i, margin+i,
		      getWidth() - 2*margin - 2*i, getHeight() - 2*margin - 2*i,
		      (int)Math.round(minAngleDegs*(1.0-critPercent)),
		      (int)Math.round(minAngleDegs*(critPercent-warnPercent)));
	    
	    // Draw the normal-color segment
	    g.setColor(normalColor);
	    g.drawArc(margin+i, margin+i,
		      getWidth() - 2*margin - 2*i, getHeight() - 2*margin - 2*i,
		      minAngleDegs,
		      (int)Math.round(minAngleDegs*(-warnPercent)));
	}
	
	for (int i=minVal;i<=maxVal+minVal;i+=littleTick) {drawTick(g,i-minVal,halfTick);}
	for (int i=minVal;i<=maxVal+minVal;i+=bigTick)    {drawTick(g,i-minVal,tickSize);}
    };
    
    /**
     * drawStrings draws the Current Value, Legend, Units, Min, and Max.
     */
    private void drawStrings(Graphics g) {
	g.setColor(getForeground());
	g.setFont(getFont());
	g.drawString(minStr, pivotLoc.x - pointerLen - myMetrics.stringWidth(minStr),
		     pivotLoc.y + minSize.height);
	g.drawString(maxStr, getLocation().x+getWidth() - 2*separation - maxSize.width,
		     pivotLoc.y + minSize.height);
	g.drawString(legend,
		     pivotLoc.x - (int)Math.round(legendSize.width/2),
		     pivotLoc.y + legendSize.height + (int)(margin*0.75));
	g.drawString(units,
		     pivotLoc.x - (int)Math.round(unitsSize.width/2),
		     pivotLoc.y + legendSize.height+(int)(margin*0.5) + unitsSize.height+separation);
        if(currVal < 0)
	    g.drawString("N/A",
			 pivotLoc.x - (int)Math.round(currValSize.width/2) - 3,
			 pivotLoc.y - (int)pointerLen/2);
	else
	    g.drawString(String.valueOf(currVal),
			 pivotLoc.x - (int)Math.round(currValSize.width/2) - 3,
			 pivotLoc.y - (int)pointerLen/2);
    };
    
    /**
     * getValColor returns the color associated with the region into which the
     * value falls.
     *	@param val		value of interest
     */
    private Color getValColor(int val) {
	if (val < warnVal) {
	    return normalColor;
	} else if (val < critVal) {
	    return warnColor;
	} else {
	    return critColor;
	}
    };
    
    /**
     * mapValToPoint returns a Point on the Scale for the specified value
     */
    private Point mapValToPoint(int val) {
	double diffRads = minAngleRads - maxAngleRads;
	double valPcnt  = (double)val/(double)maxVal;
	double valRads  = minAngleRads - diffRads*valPcnt;
	return new Point(pivotLoc.x + (int)Math.round(Math.cos(valRads)*pointerLen),
			 pivotLoc.y - (int)Math.round(Math.sin(valRads)*pointerLen));
    };
    
    /**
     * mapValToPoint returns a Point "radius" units from the Pivit for the specified value
     */
    private Point mapValToPoint(int val, int radius) {
	double diffRads = minAngleRads - maxAngleRads;
	double valPcnt  = (double)val/(double)maxVal;
	double valRads  = minAngleRads - diffRads*valPcnt;
	return new Point(pivotLoc.x + (int)Math.round(Math.cos(valRads)*radius),
			 pivotLoc.y - (int)Math.round(Math.sin(valRads)*radius));
    };
    
    /**
     * drawPointer draws the Pivot and the Pointer
     */
    private void drawPointer(Graphics g) {
	Point scalePt;
	g.setColor(getValColor(currVal));
	g.fillOval(pivotLoc.x-5, pivotLoc.y-5, 11, 11);
	if (currVal < 0)
	    scalePt = mapValToPoint(minVal);
	else
	    scalePt = mapValToPoint(currVal-minVal);
	for (int i=-penRadius;i<=penRadius;i++) {
	    g.drawLine(pivotLoc.x, pivotLoc.y+i, scalePt.x, scalePt.y+i);
	}
    };
    
    public void setSize(int w, int h) {
	super.setSize(w, h);
	partition();
	
	repaint();
	if(getHeight() < 200) {
	    setFont(new Font("Courier", Font.BOLD, 14)); 
	    if(maxVal-minVal > 150) {
		littleTick *= 2;
		bigTick *= 2;
	    }
	    if(maxVal-minVal > 300) {
		littleTick *= 3;
		bigTick *= 3;
	    }
	}
    };
    
    /**
     * paint() paints in this order:
     * <OL>
     * <LI>Background (includes Internal area)
     * <LI>Scale (includes TickMarks)
     * <LI>Strings (Legend, Units, Min, Max)
     * <LI>Pointer (inlcudes Pivot)
     * </OL>
     */
    public void paint(Graphics g) {
	if (myMetrics==null) {
	    myMetrics = getGraphics().getFontMetrics();
	    setStringMetrics();
	}
	
	drawBackground(g);
	drawScale(g);
	drawStrings(g);
	drawPointer(g);
    };
    
    public void repaint() {
	super.repaint();
    };
    
    public void repaint(long when) {
	super.repaint(when);
    };
    
    public void repaint(int x, int y, int w, int h) {
	super.repaint(x,y,w,h);
    };
    
    public void repaint(long when, int x, int y, int w, int h) {
	super.repaint(when,x,y,w,h);
    };
}
