/*
 * @(#)ImageButton.java	2.0 01/25/2005
 *
 * Copyright (c)
 * Center for Advanced Vehicular Systems
 * Mississippi State University
 *
 */
package bullybus;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.util.*;
import java.awt.Canvas.*;
/**
 * An ImageButton class
 * Turns an image into a button.
 *
 * @author 	Ron Lewis
 * @version 	2.0, 01/25/2005
 */
public class ImageButton extends java.awt.Canvas
{
   private ActionListener actionListener;  // post action events to listeners
   private boolean active;          // accepting and processing mouse events
    
   /**
    * The image to be buttonized
    */
   protected Image image; 
   /**
    * The width of the image
    */
   protected int width = -1;
   /**
    * The height of the image
    */
   protected int height = -1;
   /**
    * The action command that is fired uppon event
    */
   protected String command; 
   /**
    * Constructs an image button with i as the image and l
    * as the label and command.
    */
   public ImageButton (Image image, String command) 
   {
      this.image = image;
      setSize();
      active = true;
      this.command = command;
   }
   /**
    * Creates an image from ip, then constructs an image button
    * and uses l as the label and the command
    */
   public ImageButton (ImageProducer ip, String command) 
   {
      this.image = createImage(ip);
      setSize();
      active = true;
      this.command = command;
   }
   /**
    * Set the local deminsions to the deminsions
    * of the image.
    */
   protected synchronized void setSize () 
   {
      if (width < 0) 
         width = image.getWidth(this);
      if (height < 0) 
	 height = image.getHeight(this);
      if (width > 0 && height > 0) 
         setSize((int)(width),(int)(height));
   }
   /**
    * Sets all containers to the size of the image.
    */
   public synchronized boolean imageUpdate (Image img, int infoflags, int x, int y, int w, int h) 
   {
      if (this.width < 0 || this.height < 0) 
      {
         if ((infoflags & ImageObserver.WIDTH) != 0) 
	    this.width = w;
	 if ((infoflags & ImageObserver.HEIGHT) != 0) 
	    this.height = h;
	 if (this.width > 0 && this.height > 0) 
	 {
	    setSize((int)(this.width) , (int)(this.height));
	    Component c = this;
	    while ((c = c.getParent()) != null && ! (c instanceof Window));
	    if (c != null) 
	       ((Window)c).pack();
	 }
      }
      return super.imageUpdate(img, infoflags, x, y, width, height);
    }
    /**
     * Paints the image to the canvas
     */
    public void paint (Graphics g) 
    {
       Dimension d = getSize();
		
       if (width > 0 && height > 0 && d.width > 0 && d.height > 0) 
       {
          float hAspect = (float)d.height/height, wAspect = (float)d.width/width;
	  int h, w;
	  if (hAspect < wAspect) 
	  { 
	     h = (int)(image.getHeight(this)); 
	     w = (int)(image.getWidth(this)); 
	  }
	  else 
	  { 
	     w = (int)(image.getHeight(this));
	     h = (int)(image.getWidth(this));
	  }
	  g.drawImage(image, 0, 0,h,w,Color.white, this);
       }
				
    }
    /**
     * Returns the width of the button.
     * @return int
     */
    public int getImageWidth()
    {
       return this.width;
    }
    /**
     * Returns the height of the button.
     * @return int
     */
    public int getImageHeight()
    {
       return this.height;
    }
    /**
     * Add an action listener to the button, enables 
     * Mouse Event Mask.  Sets actionListener to listener.
     */
    public void addActionListener (ActionListener listener) {
       actionListener = AWTEventMulticaster.add (actionListener, listener);
       enableEvents(AWTEvent.MOUSE_EVENT_MASK);
    } 
    /**
     * Removes the action listener from this button.
     */
    public void removeActionListener (ActionListener listener) {
       actionListener = AWTEventMulticaster.remove (actionListener, listener);
    }
    /**
    * Sets the "active" parameter. If the cell is active then
    * it responds to mouse events.
    * 
    * @param active new "active" state value
    */
    void setActive (boolean active) {
       this.active = active;
    }
    /**
     * Sets the "image" parameter.  Changes the image to i.
     *
     * @param image change the displayed "image".
     */
    void setImage( Image image )
    {
	this.image = image;
    }
    /**
     * Resonds to MouseEvents and Fires actionEvents to the actionListener.
     * Enabled when active = true.
     */
    public void processMouseEvent (MouseEvent e) {

       if (active) {
        
          switch (e.getID()) {
	  
	  case MouseEvent.MOUSE_ENTERED:
	   Cursor cursor = new Cursor(Cursor.HAND_CURSOR );
	   setCursor(cursor);
	   break;
	  case MouseEvent.MOUSE_EXITED:
	   Cursor cursorD = new Cursor(Cursor.DEFAULT_CURSOR);
	   setCursor(cursorD);	      
	   break;
          case MouseEvent.MOUSE_PRESSED:
	  // Note: under UNIX for an unknown reason (AWT bug?) modifs is 0
	  // when the left button is pressed, but 16 when it is released!
	  // This is not the case under Win95.
	     break;
          case MouseEvent.MOUSE_RELEASED:
	     break;
          case MouseEvent.MOUSE_CLICKED:
	  // if right button is clicked
	      fireActionEvent ();
	  // if right button is clicked when in state MARKED
	
	    break; 
	}
      }
    super.processMouseEvent(e);
   }  
  /**
   * Fires an ActionEvent to the registered ActionListener
   *
   * @param command a String denoting the type of action (defaults to label, 
   * ).
   */
  void fireActionEvent () {
    if(actionListener != null) {
      actionListener.actionPerformed (new ActionEvent(
	 this, ActionEvent.ACTION_PERFORMED, command));
    }
  }

}
